/* -*- Mode: c++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 40 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is worker threads.
 *
 * The Initial Developer of the Original Code is
 *   Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Ben Turner <bent.mozilla@gmail.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef __NSDOMWORKERBASE_H__
#define __NSDOMWORKERBASE_H__

#include "nsCOMPtr.h"
#include "nsIDOMThreads.h"
#include "nsStringGlue.h"

class nsIRunnable;
class nsDOMWorkerPool;

class nsDOMWorkerBase
{
  friend class nsDOMPostMessageRunnable;

public:
  // Meant to be inherited by an nsISupports-derived class.
  NS_IMETHOD QueryInterface(REFNSIID aIID, void** aInstancePtr) = 0;
  NS_IMETHOD_(nsrefcnt) AddRef(void) = 0;
  NS_IMETHOD_(nsrefcnt) Release(void) = 0;

  PRBool IsCanceled() {
    return !!mCanceled;
  }

  PRBool IsSuspended() {
    return !!mSuspended;
  }

protected:
  nsDOMWorkerBase() : mCanceled(0), mSuspended(0) { }
  virtual ~nsDOMWorkerBase() { }

  void SetMessageListener(nsIDOMWorkerMessageListener* aListener) {
    mMessageListener = aListener;
  }

  nsIDOMWorkerMessageListener* GetMessageListener() {
    return mMessageListener;
  }

  nsresult PostMessageInternal(const nsAString& aMessage,
                               nsDOMWorkerBase* aSource);

  nsresult PostMessageInternal(const nsAString& aMessage);


  virtual void Cancel();
  virtual void Suspend();
  virtual void Resume();

  // Methods that must be implemented by a derived class.
  virtual nsresult HandleMessage(const nsAString& aMessage,
                                 nsDOMWorkerBase* aSource) = 0;
  virtual nsresult DispatchMessage(nsIRunnable* aMessage) = 0;
  virtual nsDOMWorkerPool* Pool() = 0;

private:
  nsCOMPtr<nsIDOMWorkerMessageListener> mMessageListener;
  PRInt32 mCanceled;
  PRInt32 mSuspended;
};

#endif /* __NSDOMWORKERBASE_H__ */
